@@ -17,15 +17,15 @@
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
-#include <glib.h>
-#include <gconf/gconf-client.h>
-#include <gnome-keyring.h>
-
 #include <string.h>
 #include <stdlib.h>
 
+#include <glib.h>
+#include <glib/gi18n.h>
+#include <gconf/gconf-client.h>
+#include <gnome-keyring.h>
+
 #include "gnome-keyring-manager.h"
-#include "gnome-keyring-manager-i18n.h"
 #include "gnome-keyring-manager-util.h"
 #include "gnome-keyring-manager-attribute-display.h"
 #include "gnome-keyring-manager-keyring-manager.h"
@@ -44,77 +44,26 @@ struct _GKMKeyringEditorPrivate
   GSList *gconf_cnxn_ids;
 };
 
-static void gkm_keyring_editor_class_init (GKMKeyringEditor *class);
-static void gkm_keyring_editor_init       (GKMKeyringEditor *window);
 static void gkm_keyring_editor_finalize   (GObject *object);
 
 const char *keyring_item_type_text_from_enum (GnomeKeyringItemType type);
 static void gkm_keyring_editor_setup_keyring_items_tree (GKMKeyringEditor *editor);
 
-static void open_keyring_manager_callback (GtkWidget *widget, GKMKeyringEditor *editor);
-static void add_keyring_item_callback (GtkWidget *widget, GKMKeyringEditor *editor);
-static void remove_keyring_item_callback (GtkWidget *widget, GKMKeyringEditor *editor);
-static void edit_keyring_item_callback (GtkWidget *widget, GKMKeyringEditor *editor);
-static void drag_keyring_item_secret_data_get_callback (GtkWidget *widget, GdkDragContext *context, GtkSelectionData *selection_data,
-							guint info, guint32 time, gpointer callback_data);
+/* static void add_keyring_item_callback (GtkWidget *widget,  */
+/* 				       GKMKeyringEditor *editor); */
 
-static void drag_keyring_item_data_get_callback (GtkWidget *widget, GdkDragContext *context, GtkSelectionData *selection_data,
-						 guint info, guint32 time, GKMKeyringEditor *editor);
-static void drop_keyring_item_data_recieved_callback (GtkWidget *widget, GdkDragContext *context, gint x, gint y, 
-						      GtkSelectionData *selection_data, guint info, guint32 time, GKMKeyringEditor *editor);
+static void gkm_keyring_editor_update_keyring_items (GKMKeyringEditor *self);
+static void gkm_keyring_editor_update_keyring_item_info (GKMKeyringEditor *self, 
+							 guint32 item_id, 
+							 GtkTreeIter *iter);
 
-static void close_action_callback (GtkAction *action, GKMKeyringEditor *editor);
-static void about_action_callback (GtkAction *action, GKMKeyringEditor *editor);
-static void toggle_view_column_action_callback (GtkToggleAction *action, GtkWidget *widget);
-static gboolean keyring_item_secret_changed_callback (GtkWidget *entry, GdkEventFocus *event, GKMKeyringEditor *editor);
-
-static void gkm_keyring_editor_expander_changed_callback (GtkWidget *expander, GParamSpec *spec, GKMKeyringEditor *editor);
-
-static void keyring_items_tree_context_menu_callback (GtkWidget *tree_view, GKMKeyringEditor *editor);
-static gboolean keyring_items_tree_button_press_callback (GtkWidget *tree_view, GdkEventButton *event, GKMKeyringEditor *editor);
-
-static void keyring_item_tree_selection_changed (GtkTreeSelection *selection, GKMKeyringEditor *editor);
-static void keyring_editor_update_sensitivity (GKMKeyringEditor *editor, gboolean sensitive);
-
-static void gkm_keyring_editor_update_keyring_items (GKMKeyringEditor *transient_parent);
-static void gkm_keyring_editor_update_keyring_item_info (GKMKeyringEditor *editor, guint32 item_id, GtkTreeIter *iter);
-
-static GObjectClass *parent_class = NULL;
-
-GType
-gkm_keyring_editor_get_type (void)
-{
-  static GType type = 0;
-
-  if (!type)
-    {
-      static const GTypeInfo info =
-      {
-        sizeof (GKMKeyringEditorClass),
-	NULL,		/* base_init */
-	NULL,		/* base_finalize */
-        (GClassInitFunc) gkm_keyring_editor_class_init,
-	NULL,		/* class_finalize */
-	NULL,		/* class_data */ 
-	sizeof (GKMKeyringEditor),
-	0,		/* n_preallocs */
-	(GInstanceInitFunc) gkm_keyring_editor_init,
-	0
-      };
-
-      type = g_type_register_static (G_TYPE_OBJECT, "GKMKeyringEditor", &info, 0);
-    }
-
-  return type;
-}
+G_DEFINE_TYPE (GKMKeyringEditor, gkm_keyring_editor, G_TYPE_OBJECT);
 
 static void
-gkm_keyring_editor_class_init (GKMKeyringEditor *class)
+gkm_keyring_editor_class_init (GKMKeyringEditorClass *class)
 {
   GObjectClass *gobject_class = G_OBJECT_CLASS (class);
 
-  parent_class = g_type_class_peek_parent (class);
-
   gobject_class->finalize = gkm_keyring_editor_finalize;
 }
 
@@ -139,7 +88,7 @@ gkm_keyring_editor_finalize (GObject *object)
  
   g_free (editor->priv);
 
-  G_OBJECT_CLASS (parent_class)->finalize (object);
+  G_OBJECT_CLASS (gkm_keyring_editor_parent_class)->finalize (object);
 }
 
 GObject *
@@ -270,21 +219,21 @@ gkm_keyring_editor_get_keyring_name (GKMKeyringEditor *self)
  * Create a new keyring item.
  */
 
-static void
-add_keyring_item_done_callback (GnomeKeyringResult result,
-				guint32 val G_GNUC_UNUSED,
-				GKMKeyringEditor *editor)
-{
-  if (result != GNOME_KEYRING_RESULT_OK)
-    {
-      //TODO: This should be a signal
-      complain_about_gnome_keyring_bad_result (NULL, result);
+/* static void */
+/* add_keyring_item_done_callback (GnomeKeyringResult result, */
+/* 				guint32 val G_GNUC_UNUSED, */
+/* 				GKMKeyringEditor *editor) */
+/* { */
+/*   if (result != GNOME_KEYRING_RESULT_OK) */
+/*     { */
+/*       //TODO: This should be a signal */
+/*       complain_about_gnome_keyring_bad_result (NULL, result); */
 
-      return;
-    } 
+/*       return; */
+/*     }  */
 
-  gkm_keyring_editor_update_keyring_items (editor); 
-}
+/*   gkm_keyring_editor_update_keyring_items (editor);  */
+/* } */
 
 /* static void */
 /* add_keyring_item_action_callback (GKMNewItemDialog *dialog, */
@@ -347,160 +296,170 @@ add_keyring_item_done_callback (GnomeKeyringResult result,
 /*   gtk_window_present (GTK_WINDOW (dialog)); */
 /* } */
 
-/* typedef struct _SaveItemCallbackData */
-/* { */
-/*   GKMKeyringEditor    *editor; */
-/*   GKMAttributeEditor  *attribute_editor; */
-/*   guint32	       item_id; */
-/*   GtkTreeRowReference *row; */
-/* } SaveItemCallbackData; */
+/***********************************************
+ * Setting key attributes.
+ */
 
-/* static void */
-/* save_item_callback_data_free (SaveItemCallbackData *data) */
-/* { */
-/*   gtk_tree_row_reference_free (data->row); */
-/*   g_free (data); */
-/* } */
+typedef struct _SaveItemCallbackData
+{
+  GKMKeyringEditor *editor;
+  guint32	    item_id;
+  GtkTreeIter       iter;
+} SaveItemCallbackData;
 
+static void
+attributes_saved_callback (GnomeKeyringResult result,
+			   SaveItemCallbackData *data)
+{
+  if (result != GNOME_KEYRING_RESULT_OK)
+    {
+      complain_about_gnome_keyring_bad_result (NULL, result);
+      return;
+    }
 
-/* static void */
-/* attributes_saved_callback (GnomeKeyringResult result, */
-/* 			   SaveItemCallbackData *data) */
-/* { */
-/*   GtkTreePath *path; */
-/*   GtkTreeIter iter; */
-/*   GtkTreeSelection *selection; */
+  gkm_keyring_editor_update_keyring_item_info (data->editor, data->item_id, &data->iter);
 
-/*   if (result != GNOME_KEYRING_RESULT_OK) */
-/*     { */
-/*       complain_about_gnome_keyring_bad_result (NULL, result); */
-/*       return; */
-/*     } */
-
-/*   if (!gtk_tree_row_reference_valid (data->row)) */
-/*     { */
-/*       g_warning (_("A row disapeared while we were waiting for the data...")); */
-
-/*       return; */
-/*     } */
-
-/*   path = gtk_tree_row_reference_get_path (data->row); */
-/*   gtk_tree_model_get_iter (GTK_TREE_MODEL (data->editor->priv->keyring_items), &iter, path); */
-/*   gtk_tree_path_free (path); */
-
-/*   gkm_keyring_editor_update_keyring_item_info (data->editor, data->item_id, &iter); */
+  /* TODO: This should be in main_ui */
 /*   selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(data->editor->priv->items_tree_view)); */
 /*   keyring_item_tree_selection_changed (selection, data->editor); */
-/* } */
+}
 
-/* static void */
-/* acl_saved_callback (GnomeKeyringResult result, */
-/* 		    SaveItemCallbackData *data) */
-/* { */
-/*   if (result != GNOME_KEYRING_RESULT_OK) */
-/*     { */
-/*       complain_about_gnome_keyring_bad_result (NULL, result); */
-/*       return; */
-/*     } */
+void
+gkm_keyring_editor_set_attributes_list (GKMKeyringEditor *self,
+					GtkTreeIter       iter,
+					GnomeKeyringAttributeList *attributes)
+{
+  SaveItemCallbackData *data;
+
+  data = g_new0 (SaveItemCallbackData, 1);
+
+  data->editor = self;
+  data->iter = iter;
+
+  gtk_tree_model_get (GTK_TREE_MODEL (self->priv->keyring_items), &iter,
+		      EDITOR_COLUMN_ID, &data->item_id, -1);
+
+  gnome_keyring_item_set_attributes (data->editor->priv->keyring_name, 
+				     data->item_id, 
+				     attributes,
+				     (GnomeKeyringOperationDoneCallback) 
+				     attributes_saved_callback,
+				     data, g_free);
+
+  gnome_keyring_attribute_list_free (attributes);
+}
+
+static void
+acl_saved_callback (GnomeKeyringResult result,
+		    SaveItemCallbackData *data G_GNUC_UNUSED)
+{
+  if (result != GNOME_KEYRING_RESULT_OK)
+    {
+      complain_about_gnome_keyring_bad_result (NULL, result);
+      return;
+    }
 
 /*   if (!gtk_tree_row_reference_valid (data->row)) */
 /*     { */
-/*       g_warning (_("A row disapeared while we were waiting for the data...")); */
+/*       g_warning (_("A row disappeared while we were waiting for the data...")); */
 
 /*       return; */
 /*     } */
-/* } */
-
-/* void */
-/* save_attributes (SaveItemCallbackData *data) */
-/* { */
-/*   GnomeKeyringAttributeList *attribute_list; */
-
-/*   attribute_list = gkm_attribute_editor_get_attribute_list (data->attribute_editor); */
-
-/*   gnome_keyring_item_set_attributes (data->editor->priv->keyring_name, data->item_id, attribute_list, */
-/* 				     (GnomeKeyringOperationDoneCallback) attributes_saved_callback, */
-/* 				     data, NULL); */
-
-/*   gnome_keyring_attribute_list_free (attribute_list); */
-/* } */
-
-/* void */
-/* save_acl (SaveItemCallbackData *data) */
-/* { */
-/*   GList *acl; */
-
-/*   acl = gkm_attribute_editor_get_acl (data->attribute_editor); */
-
-/*   gnome_keyring_item_set_acl (data->editor->priv->keyring_name, data->item_id, acl, */
-/* 			      (GnomeKeyringOperationDoneCallback) acl_saved_callback, */
-/* 			      data, (GDestroyNotify) save_item_callback_data_free); */
-
-/*   gnome_keyring_acl_free (acl); */
-/* } */
+}
 
 
+void 
+gkm_keyring_editor_set_acl (GKMKeyringEditor *self,
+			    GtkTreeIter       iter,
+			    GList            *acl)
+{
+  SaveItemCallbackData *data;
 
-/* static gboolean */
-/* edit_keyring_item_closed_callback (GtkWidget *widget, GdkEvent *event G_GNUC_UNUSED, SaveItemCallbackData *data) */
-/* { */
-/*   if (data->attribute_editor->attributes_modified == TRUE) */
-/*     { */
-/*       save_attributes (data); */
-/*     } */
-/*   if (data->attribute_editor->acl_modified == TRUE) */
-/*     { */
-/*       save_acl (data); */
-/*     } */
+  data = g_new0 (SaveItemCallbackData, 1);
 
+  data->editor = self;
+  data->iter = iter;
 
-/*   gtk_widget_destroy (widget); */
+  gtk_tree_model_get (GTK_TREE_MODEL (self->priv->keyring_items), &iter, 
+		      EDITOR_COLUMN_ID, &data->item_id, -1);
 
-/*   return FALSE; */
-/* } */
+  gnome_keyring_item_set_acl (self->priv->keyring_name, data->item_id, acl,
+			      (GnomeKeyringOperationDoneCallback) 
+			      acl_saved_callback,
+			      data, g_free);
 
+  gnome_keyring_acl_free (acl);
+}
 
+/********************************************************
+ * Change the name of a keyring item.
+ */
 
+typedef struct _RenameItemCallbackData
+{
+  GKMKeyringEditor *editor;
+  GtkTreeIter	    iter;
+  guint32           item_id;
+  gchar            *new_name;
+} RenameItemCallbackData;
 
-/* static void */
-/* edit_keyring_item_callback (GtkWidget        *button G_GNUC_UNUSED, */
-/*    	                    GKMKeyringEditor *editor G_GNUC_UNUSED) */
-/* { */
-/*   GtkTreeSelection *selection; */
-/*   GtkTreeModel *model; */
-/*   GtkTreeIter iter; */
+static void
+rename_item_done_callback (GnomeKeyringResult        result,
+			   RenameItemCallbackData   *data)
+{
+  if (result != GNOME_KEYRING_RESULT_OK)
+    {
+      complain_about_gnome_keyring_bad_result (NULL, result);
+      return;
+    }
 
-/*   g_return_if_fail (GKM_IS_KEYRING_EDITOR (editor)); */
+  gtk_list_store_set (data->editor->priv->keyring_items, &data->iter,
+		      EDITOR_COLUMN_NAME, data->new_name, -1);
+}
 
-/*   selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (editor->priv->items_tree_view)); */
+static void
+rename_item_get_info_callback (GnomeKeyringResult      result,
+			       GnomeKeyringItemInfo   *info,
+			       RenameItemCallbackData *data)
+{
+  if (result != GNOME_KEYRING_RESULT_OK)
+    {
+      complain_about_gnome_keyring_bad_result (NULL, result);
+      g_free (data);
+      return;
+    }
   
-/*   if (gtk_tree_selection_get_selected (selection, &model, &iter)) */
-/*     { */
-/*       GtkTreePath *tmp_path; */
-/*       SaveItemCallbackData *data; */
-/*       GtkWidget *attribute_editor; */
-/*       guint item_id; */
-/*       char *item_name; */
+  gnome_keyring_item_info_set_display_name (info, data->new_name);
+  
+  gnome_keyring_item_set_info (data->editor->priv->keyring_name, 
+			       data->item_id, info,
+			       (GnomeKeyringOperationDoneCallback)
+			       rename_item_done_callback,
+			       data, g_free);
+}
 
-/*       gtk_tree_model_get (model, &iter, EDITOR_COLUMN_ID, &item_id, -1); */
-/*       gtk_tree_model_get (model, &iter, EDITOR_COLUMN_NAME, &item_name, -1); */
+void 
+gkm_keyring_editor_set_key_name (GKMKeyringEditor *self,
+				 GtkTreeIter       iter,
+				 const gchar      *new_name)
+{
+  RenameItemCallbackData *data;
 
-/*       attribute_editor = gkm_attribute_editor_new (editor->priv->keyring_name, item_id, item_name); */
+  data = g_new0 (RenameItemCallbackData, 1);
 
-/*       data = g_new0 (SaveItemCallbackData, 1); */
-/*       data->editor = editor; */
-/*       data->attribute_editor = GKM_ATTRIBUTE_EDITOR (attribute_editor); */
-/*       data->item_id = item_id; */
-/*       tmp_path = gtk_tree_model_get_path (model, &iter); */
-/*       data->row = gtk_tree_row_reference_new (model, tmp_path); */
-/*       gtk_tree_path_free (tmp_path); */
+  data->editor = self;
+  data->iter = iter;
+  data->new_name = g_strdup (new_name);
 
-/*       g_signal_connect (attribute_editor, "delete-event", G_CALLBACK (edit_keyring_item_closed_callback), data); */
+  gtk_tree_model_get (GTK_TREE_MODEL (self->priv->keyring_items), &iter, 
+		      EDITOR_COLUMN_ID, &data->item_id, -1);
+
+  gnome_keyring_item_get_info (self->priv->keyring_name, data->item_id,
+			       (GnomeKeyringOperationGetItemInfoCallback)
+			       rename_item_get_info_callback,
+			       data, NULL);
+}
 
-/*       gtk_widget_show (GTK_WIDGET (attribute_editor)); */
-/*       g_free (item_name);  */
-/*     } */
-/* } */
 
 /********************************************************
  * Delete a keyring item
@@ -599,89 +558,77 @@ change_secret_callback_data_free (ChangeSecretCallbackData *data)
   g_free (data);
 }
 
-/* static void */
-/* change_secret_set_info_callback (GnomeKeyringResult        result, */
-/* 				 ChangeSecretCallbackData *data) */
-/* { */
-/*   if (result != GNOME_KEYRING_RESULT_OK) */
-/*     { */
+static void
+change_secret_set_info_callback (GnomeKeyringResult        result,
+				 ChangeSecretCallbackData *data)
+{
+  if (result != GNOME_KEYRING_RESULT_OK)
+    {
 /*       gtk_label_set_text (GTK_LABEL (data->editor->priv->secret_field), data->old_secret); */
 /*       gkm_keyring_editor_set_secret (data->editor, data->old_secret); */
       
-/*       complain_about_gnome_keyring_bad_result (GTK_WINDOW (data->editor), result); */
-/*       return; */
-/*     } */
+      complain_about_gnome_keyring_bad_result (GTK_WINDOW (data->editor), result);
+      return;
+    }
 
-/*    gtk_list_store_set (data->editor->priv->keyring_items, &data->iter, EDITOR_COLUMN_SECRET, data->new_secret, -1); */
-/* } */
+   gtk_list_store_set (data->editor->priv->keyring_items, &data->iter, 
+		       EDITOR_COLUMN_SECRET, data->new_secret, -1);
+}
 
-/* static void */
-/* change_secret_get_info_callback (GnomeKeyringResult        result, */
-/* 			         GnomeKeyringItemInfo     *info, */
-/* 			         ChangeSecretCallbackData *data) */
-/* { */
-/*   if (result != GNOME_KEYRING_RESULT_OK) */
-/*     { */
+static void
+change_secret_get_info_callback (GnomeKeyringResult        result,
+			         GnomeKeyringItemInfo     *info,
+			         ChangeSecretCallbackData *data)
+{
+  if (result != GNOME_KEYRING_RESULT_OK)
+    {
 /*       gkm_keyring_editor_set_secret (data->editor, data->old_secret); */
  
-/*       complain_about_gnome_keyring_bad_result (GTK_WINDOW (data->editor), result); */
-/*       return; */
-/*     }  */
+      complain_about_gnome_keyring_bad_result (GTK_WINDOW (data->editor), result);
+      return;
+    }
 
-/*   gnome_keyring_item_info_set_secret (info, data->new_secret); */
+  gnome_keyring_item_info_set_secret (info, data->new_secret);
 
-/*   gnome_keyring_item_set_info (data->editor->priv->keyring_name, */
-/*   			       data->item_id, */
-/*   			       info, */
-/* 			       (GnomeKeyringOperationDoneCallback) change_secret_set_info_callback, */
-/* 			       change_secret_callback_data_clone (data), */
-/* 			       (GDestroyNotify) change_secret_callback_data_free); */
-/* } */
+  gnome_keyring_item_set_info (data->editor->priv->keyring_name,
+  			       data->item_id,
+  			       info,
+			       (GnomeKeyringOperationDoneCallback) change_secret_set_info_callback,
+			       change_secret_callback_data_clone (data),
+			       (GDestroyNotify) change_secret_callback_data_free);
+}
 
-/* static gboolean */
-/* keyring_item_secret_changed_callback (GtkWidget *entry G_GNUC_UNUSED, */
-/* 				      GdkEventFocus *event G_GNUC_UNUSED, */
-/* 				      GKMKeyringEditor *editor) */
-/* { */
-/*   GtkTreeSelection *selection; */
-/*   GtkTreeModel     *model; */
-/*   GtkTreeIter       iter; */
-/*   gchar            *new_secret; */
-/*   gchar	           *secret; */
-/*   guint32	    item_id; */
+void 
+gkm_keyring_editor_set_key_secret      (GKMKeyringEditor *self,
+					GtkTreeIter       iter,
+					const gchar      *new_secret)
+{
+  guint32  item_id;
+  gchar   *secret;
 
-/*   g_return_val_if_fail (GKM_IS_KEYRING_EDITOR (editor), FALSE); */
+  gtk_tree_model_get (GTK_TREE_MODEL (self->priv->keyring_items), &iter, 
+		      EDITOR_COLUMN_ID, &item_id, 
+		      EDITOR_COLUMN_SECRET, &secret, -1);
 
-/*   selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (editor->priv->items_tree_view)); */
-  
-/*   if (gtk_tree_selection_get_selected (selection, &model, &iter)) */
-/*     { */
-/*       gtk_tree_model_get (model, &iter, EDITOR_COLUMN_ID, &item_id, EDITOR_COLUMN_SECRET, &secret, -1); */
-/*       new_secret = gkm_keyring_editor_get_secret (editor); */
-
-/*       if (strcmp (new_secret, secret) != 0) */
-/*         { */
-/*         ChangeSecretCallbackData *data; */
-
-/*         data = g_new0 (ChangeSecretCallbackData, 1); */
-/* 	data->editor = editor; */
-/* 	data->item_id = item_id; */
-/* 	data->old_secret = g_strdup (secret); */
-/* 	data->new_secret = new_secret; */
-/* 	data->iter = iter; */
-	
-/* 	gnome_keyring_item_get_info (editor->priv->keyring_name, */
-/* 				     item_id, */
-/* 				     (GnomeKeyringOperationGetItemInfoCallback) change_secret_get_info_callback, */
-/* 				     data,  */
-/* 				     (GDestroyNotify) change_secret_callback_data_free);  */
-/* 	} */
-
-/*       g_free (secret); */
-/*     } */
-  
-/*   return FALSE; */
-/* } */
+  if (!g_str_equal (new_secret, secret))
+    {
+      ChangeSecretCallbackData *data;
+      
+      data = g_new0 (ChangeSecretCallbackData, 1);
+      data->editor = self;
+      data->item_id = item_id;
+      data->old_secret = g_strdup (secret);
+      data->new_secret = g_strdup (new_secret);
+      data->iter = iter;
+      
+      gnome_keyring_item_get_info (self->priv->keyring_name,
+				   item_id,
+				   (GnomeKeyringOperationGetItemInfoCallback) 
+				   change_secret_get_info_callback,
+				   data,
+				   (GDestroyNotify) change_secret_callback_data_free);
+    }
+}
 
 /********************************************************
  * Get the attributes of the selected key.
@@ -739,6 +686,60 @@ gkm_keyring_editor_get_attributes_list (GKMKeyringEditor *self,
 				     data, g_free);
 }
 
+/********************************************************
+ * Get the acl of the selected key.
+ */
+
+typedef struct _GetAclCallbackData
+{
+  GKMKeyringEditor *editor;
+  GKMKeyringEditorGetAclCallback callback_func;
+  gpointer  callback_data;
+} GetAclCallbackData;
+
+static void
+gkm_keyring_editor_get_acl_callback (GnomeKeyringResult  result,
+				     GSList             *acl,
+				     GetAclCallbackData *data)
+{
+  if (result != GNOME_KEYRING_RESULT_OK)
+    {
+      /* fixme should be passing gerrors back, instead */
+      complain_about_gnome_keyring_bad_result (NULL, result);
+      return;
+    }
+
+  (*data->callback_func) (data->callback_data, acl);
+}
+
+void 
+gkm_keyring_editor_get_acl (GKMKeyringEditor *self,
+			    GtkTreeIter       iter,
+			    GKMKeyringEditorGetAclCallback callback,
+			    gpointer          user_data)
+{
+  GetAclCallbackData *data;
+  guint32 item_id;
+
+  g_return_if_fail (GKM_IS_KEYRING_EDITOR (self));
+
+  data = g_new0 (GetAclCallbackData, 1);
+  data->editor = self;
+  data->callback_func = callback;
+  data->callback_data = user_data;
+  
+  
+  gtk_tree_model_get (GTK_TREE_MODEL (self->priv->keyring_items), &iter, 
+		      EDITOR_COLUMN_ID, &item_id, 
+		      -1);
+  
+  gnome_keyring_item_get_acl (self->priv->keyring_name,
+			      item_id,
+			      (GnomeKeyringOperationGetListCallback) 
+			      gkm_keyring_editor_get_acl_callback,
+			      data, g_free);
+}
+
 /* static void */
 /* keyring_editor_update_sensitivity (GKMKeyringEditor *editor,  */
 /*                                    gboolean sensitive) */
@@ -877,11 +878,12 @@ update_keyring_items_worker_callback (GnomeKeyringResult  result,
 }
 
 static void
-gkm_keyring_editor_update_keyring_items (GKMKeyringEditor *transient_parent)
+gkm_keyring_editor_update_keyring_items (GKMKeyringEditor *self)
 {
-  gnome_keyring_list_item_ids (transient_parent->priv->keyring_name,
-  			       (GnomeKeyringOperationGetListCallback) update_keyring_items_worker_callback, 
-                               transient_parent, NULL);
+  gnome_keyring_list_item_ids (self->priv->keyring_name,
+  			       (GnomeKeyringOperationGetListCallback) 
+			       update_keyring_items_worker_callback, 
+                               self, NULL);
 }
  
 /********************************************************
@@ -921,7 +923,7 @@ update_keyring_item_info_worker_callback (GnomeKeyringResult             result,
 
   if (!gtk_tree_row_reference_valid (data->row))
     {
-      g_warning (_("A row disapeared while we were waiting for the data..."));
+      g_warning (_("A row disappeared while we were waiting for the data..."));
 
       return;
     }
@@ -950,7 +952,7 @@ update_keyring_item_info_worker_callback (GnomeKeyringResult             result,
 
 /**
  * gkm_keyring_editor_update_keyring_item_info:
- * @editor: a #GKMKeyringEditor.
+ * @self: a #GKMKeyringEditor.
  * @item_id: the id of the item whose info is to be updated.
  * @iter: a #GtkTreeIter pointing to @keyring in the keyrings @GtkListStore, 
  * or %NULL.
@@ -959,18 +961,18 @@ update_keyring_item_info_worker_callback (GnomeKeyringResult             result,
  **/
 
 static void
-gkm_keyring_editor_update_keyring_item_info (GKMKeyringEditor *editor, 
+gkm_keyring_editor_update_keyring_item_info (GKMKeyringEditor *self, 
 		 			     guint32           item_id,
 					     GtkTreeIter      *iter)
 {
   UpdateKeyringItemCallbackData *data;
 
   data = g_new0 (UpdateKeyringItemCallbackData, 1);
-  data->editor = editor;
+  data->editor = self;
 
   if (iter == NULL)
     {
-      if (!search_keyring_item (editor, item_id, NULL, NULL, &data->row))
+      if (!search_keyring_item (self, item_id, NULL, NULL, &data->row))
         {
           g_warning ("gkm_keyring_editor_update_keyring_item_info: Tried to update the information of a keyring item we don't know about");
         }
@@ -979,13 +981,20 @@ gkm_keyring_editor_update_keyring_item_info (GKMKeyringEditor *editor,
     {
       GtkTreePath *path;
 
-      path = gtk_tree_model_get_path (GTK_TREE_MODEL (editor->priv->keyring_items), iter);
-      data->row = gtk_tree_row_reference_new (GTK_TREE_MODEL (editor->priv->keyring_items), path);
+      path = 
+	gtk_tree_model_get_path (GTK_TREE_MODEL (self->priv->keyring_items),
+				 iter);
+      data->row = 
+	gtk_tree_row_reference_new (GTK_TREE_MODEL (self->priv->keyring_items),
+				    path);
       gtk_tree_path_free (path);
     }
   
-  gnome_keyring_item_get_info (editor->priv->keyring_name,
+  gnome_keyring_item_get_info (self->priv->keyring_name,
   			       item_id,
-                               (GnomeKeyringOperationGetItemInfoCallback) update_keyring_item_info_worker_callback, 
-                               data, (GDestroyNotify) update_keyring_item_callback_data_free);
+                               (GnomeKeyringOperationGetItemInfoCallback) 
+			       update_keyring_item_info_worker_callback, 
+                               data, 
+			       (GDestroyNotify) 
+			       update_keyring_item_callback_data_free);
 }
